<!-- markdownlint-disable MD034 -->
# vSphere CSI Driver - v2.0.1 release

## New Feature

- There is no new feature released in v2.0.1.  v2.0.1 is the patch release to fix some critical issues observed in [v2.0.0](v2.0.0.md) release.
  
## Notable Changes

- Fixed backward compatibility issue with vSphere 67u3 release. [#409](https://github.com/kubernetes-sigs/vsphere-csi-driver/pull/409)
- Fixed race between detach volume and delete volume caused by bug in the `external-provisioner`. [#438](https://github.com/kubernetes-csi/external-provisioner/pull/438)
  
## Deployment files

- https://github.com/kubernetes-sigs/vsphere-csi-driver/tree/master/manifests/v2.0.1

## Kubernetes Release

- Minimum: 1.17
- Maximum: 1.19

## Supported sidecar containers versions

- csi-provisioner - v2.0.0
- csi-attacher - v2.0.0
- csi-resizer - v0.3.0
- livenessprob - v1.1.0
- csi-node-driver-registrar - v1.2.0

## Known Issues

### vSphere CSI Driver issues

1. When the static persistent volume is re-created with the same PV name, volume is not getting registered as a container volume with vSphere.
   - Impact: attach/delete can not be performed on the such Persistent Volume.
   - Workaround: wait for 1 hour before re-creating static persistent volume using the same name.
2. Metadata syncer container deletes the volume physically from the datastore when Persistent Volumes with `Bound` status and reclaim policy `Delete` is deleted by the user when `StorageObjectInUseProtection` is disabled on Kubernetes Cluster.
   - Impact: Persistent Volumes Claim goes in the lost status. Volume can not be recovered.
   - Workaround: Do not disable `StorageObjectInUseProtection` and attempt to delete Persistent Volume directly without deleting PVC.
3. deployment yaml uses `hostPath` volume in the CSI driver deployment for unix domain socket path.
   - Impact: when the controller Pod does not have access to the file system on the node VM, driver fails to create socket file and thus does not come up.
   - Workaround: use `emptydir` volume instead of `hostPath` volume.
4. Volume expansion might fail when it is called with pod creation simultaneously.
    - Impact: Users can resize the PVC and create a pod using that PVC simultaneously. In this case, pod creation might be completed first using the PVC with original size. Volume expansion will fail because online resize is not supported in vSphere 7.0 Update1.
    - Workaround: Wait for the PVC to reach FileVolumeResizePending condition before attaching a pod to it.
5. Deleting PV before deleting PVC, leaves orphan volume on the datastore.
   - Impact: Orphan volumes remain on the datastore, and admin needs to delete those volumes manually using `govc` command.
   - Upstream issue is tracked at: https://github.com/kubernetes-csi/external-provisioner/issues/546
   - Workaround:
      - No workaround. User should not attempt to delete PV which is bound to PVC. User should only delete a PV if they know that the underlying volume in the storage system is gone.
      - If user has accidentally left orphan volumes on the datastore by not following the guideline, and if user has captured the volume handles or First Class Disk IDs of deleted PVs, storage admin can help delete those volumes using `govc disk.rm <volume-handle/FCD ID>` command.
6. When multiple PVCs and Pods with the same name present on the Cluster, and for any reason, Volume gets de-registered or lost from vCenter CNS Database, Syncer does not re-register volume back.
   - Impact: Volume will not re-appear on the CNS UI. If volume needs to be detached and attached to newer node, it will not happen.
   - Workaround:
      - This issue is fixed in [v2.1.0](./v2.1.0.md) release. Please consider upgrading driver to v2.1.0

### Kubernetes issues

1. Filesystem resize is skipped if the original PVC is deleted when FilesystemResizePending condition is still on the PVC, but PV and its associated volume on the storage system are not deleted due to the Retain policy.
   - Issue: https://github.com/kubernetes/kubernetes/issues/88683
   - Impact: User may create a new PVC to statically bind to the undeleted PV. In this case, the volume on the storage system is resized but the filesystem is not resized accordingly. User may try to write to the volume whose filesystem is out of capacity.
   - Workaround: User can log into the container to manually resize the filesystem.
2. Volume associated with a Statefulset cannot be resized
   - Issue: https://github.com/kubernetes/enhancements/pull/660
   - Impact: User cannot resize volume in a StatefulSet.
   - Workaround: If the statefulset is not managed by an operator, there is a slightly risky workaround which the user can use on their own discretion depending upon their use case. Please refer to https://serverfault.com/questions/955293/how-to-increase-disk-size-in-a-stateful-set for more details.
3. Recover from volume expansion failure.
   - Impact: If volume expansion fails because storage system does not support it, there is no way to recover.
   - Issue: https://github.com/kubernetes/enhancements/pull/1516
   - Workaround: None

## vSphere issues

1. CNS file volume has a limitation of 8K for metadata.
   - Impact: It is quite possible that we will not be able to push all the metadata to CNS file share as we need support a max of 64 clients per file volume.
   - Workaround: None. This is vSphere limitation.
